clear all, clc, close all

%% ************************************************************************
%  * Initialization of controller dependent variables
%  ************************************************************************
ProblemParms = SetupProblemParms;
Xpid = [0;0];
U = [0;0];
time_old = 0;
time = 0;

%% ************************************************************************
%  * Write htc files for HAWC2
%  ************************************************************************
% Close existing sessions of HAWC2
% % % if isunix
% % %     if system('killall -9 hawc2MB.exe');
% % %         pause(20);
% % %     end
% % % else
    system('taskkill.exe /f /im hawc2mb.exe');
    system('taskkill.exe /f /im cmd.exe');
% % % end

% Simulation time options
Tstart = 0; % From what time results should be saved
Tend = 100; % Simulation length
Ts = 0.025; % Sample time

% Choose simulation type
SimulationType = 'script'; % TCP/IP
% SimulationType = 'simulink'; % TCP/IP
% SimulationType = 'compiledll'; % compiled dll

% Write simulation setup
fid=fopen('../hawc2/htc/simulation_simulation.htc','w');
fprintf(fid,'time_stop  %1.4e ; \n',Tend+Ts);
fprintf(fid,'solvertype   1 ;    (newmark)  \n');
fprintf(fid,'on_no_convergence continue ; \n');
fprintf(fid,'logfile ./log/nrelmatlab.log ; \n');
fprintf(fid,'; \n');
fprintf(fid,'begin NEWMARK ; \n');
fprintf(fid,'  deltat    %1.4e ; \n',Ts);
fprintf(fid,'end NEWMARK ; \n');
fprintf(fid,'exit ; \n');
fclose(fid);

% Write results saving options
ResultsFileName = 'HAW2matlab';
fid=fopen('../hawc2/htc/simulation_results_length.htc','w');
fprintf(fid,'filename ./res/%s ; \n',ResultsFileName);
fprintf(fid,'time %1.8e %1.8e ; \n',Tstart,Tend);
fprintf(fid,'exit ; \n');
fclose(fid);

fid=fopen('../hawc2/htc/simulation_control.htc','w');
switch SimulationType
    case {'script','simulink'}
        TCPserverType = 'tcplink_delay';
        % TCPserverType = 'tcplink';
        TCPportnumber = 1039;
        
        % Write TCPserver type with or without delay
        fprintf(fid,'filename ./tcpip/TCPserver.dll ; \n');
        fprintf(fid,'dll_subroutine %s ; \n',TCPserverType);
        fprintf(fid,'init_string %s ; port number (optional, default is 1239)\n',num2str(TCPportnumber));
    case 'compiledll'
        fprintf(fid,'filename ./control/pid_wt_hawc2.dll ; \n');
        fprintf(fid,'dll_subroutine pid_wt ; \n');
end
fprintf(fid,'exit ; \n');
fclose(fid);


switch SimulationType
    case {'compiledll'}
        mcc -v -B csharedlib:pid_wt_matlab pid_wt.m pid_wt_start.m
        system('copy pid_wt_matlab.dll ..\hawc2\');
        
        mbuild -v pid_wt_hawc2.c pid_wt_matlab.lib pid_wt_hawc2.exports
        system('copy pid_wt_hawc2.dll ..\hawc2\control\');
end

%% ************************************************************************
%  * Start HAWC2 and run simulation
%  ************************************************************************
% tic-toc measures computation time of simulation
tic

% Open HAWC2
cd('../hawc2');
% !hawc2MB.exe ./htc/HAWC2matlab.htc &
% system('hawc2MB.exe ./htc/HAWC2matlab.htc');
if isunix
    system('wine hawc2MB.exe ./htc/HAWC2matlab.htc')
%     !wine hawc2MB.exe ./htc/HAWC2matlab.htc &
else
    !hawc2MB.exe ./htc/HAWC2matlab.htc &
%     system('wine hawc2MB.exe ./htc/HAWC2matlab.htc')
end
cd('../matlab');

switch SimulationType
    case {'script','simulink'}
        % CONNECTS TO HAWC2 TCP-HOST
        TcpIpConnected = 0;
        for i = 1:100
            if TcpIpConnected
                break,
            end
            pause(.1);
            try
                JTCPOBJ = jtcp('REQUEST','localhost',TCPportnumber);
                TcpIpConnected = 1;
            catch TryTcpIpConnect
                TcpIpConnected = 0;
            end
        end
        
        
        switch SimulationType
            case 'script'
                HAWC2matlabScript
            case 'simulink'
                sim('HAW2test',[0 Tend])
        end
        % DISCONNECT FROM THE HAWC2 TCP-HOST
        jtcp('CLOSE',JTCPOBJ)
        
        % Close existing sessions of HAWC2
%         system('taskkill.exe /f /im hawc2mb.exe');
%         system('taskkill.exe /f /im cmd.exe');
        
        toc
        

end

%% ************************************************************************
%  * Print results from HAWC2 generated result file
%  ************************************************************************
% % % [sig1,Freq1,Time1,Flag1,Binary1] = ...
% % %     ReadHawc2(['..\hawc2\res\' ResultsFileName]);
% % % 
% % % figure
% % % plot(sig1(:,1),-sig1(:,3)*ProblemParms.Ng);
