function [U_old,X_old,time_old,Ts] = pid_wt(U_old,X_old,time_old,Ts,omega_g,time,ProblemParms)
% U - (1) Pitch angle, (2) generator torque
% X - (1) Filtered generator speed, (2) integral error state

Ei = X_old(1);
omega_g_filter = X_old(2);

% Lowpass filter for generator speed
CornerFreq= 0.25*2*pi;
Alpha = exp( ( -Ts )*CornerFreq );
omega_g_filter = ( 1.0 - Alpha )*omega_g + Alpha*omega_g_filter;

% Speed error
Ep = omega_g_filter - ProblemParms.omega_g_nom;
% Integrated speed error
Ei = Ei + Ep*Ts;

Kgs = 1/(1+(pi/180*(U_old(1)-ProblemParms.th_min))/ProblemParms.Kk)*180/pi;

% Limit of generator speed where optimal tracking ends
omega_g_high = ProblemParms.omega_g_nom*.95;

% Generator torque control
Qh = ProblemParms.KII * omega_g_high^2/ProblemParms.Ng^3;
Qnom = ProblemParms.Pnom/ProblemParms.omega_g_nom;

if U_old(1) <= ProblemParms.th_min+0.1 % Below rated wind speed
    %     Ei = 0;
    if omega_g_filter <= omega_g_high
        
        U(2,1) = ProblemParms.KII * omega_g_filter^2/ProblemParms.Ng^3;
    else
        U(2,1) = Qh + (Qnom-Qh)*(omega_g_filter-omega_g_high)/(ProblemParms.omega_g_nom-omega_g_high);
    end
else % Above rated wind speed
    U(2,1) = ProblemParms.Pnom/omega_g_filter;
end

% PI controller for pitch angle
U(1,1) = Kgs*(ProblemParms.Kp*Ep + ProblemParms.Ki*Ei);

% Saturation limits of control signals
Umax = [ProblemParms.th_max ProblemParms.Qgmax]';
Umin = [ProblemParms.th_min ProblemParms.Qgmin]';
dUmax = [ProblemParms.dthmax ProblemParms.dQgmax]';
dUmin = [ProblemParms.dthmin ProblemParms.dQgmin]';
% Saturate control signal
U = min(max(U,Umin),Umax);
% Saturate control signal rate
dU = min(max((U-U_old)/Ts,dUmin),dUmax);
% Calc. saturated control signal
U = U_old+dU*Ts;

% Integrator anti-windup
Ei = (U(1) - Kgs*ProblemParms.Kp*Ep)/(Kgs*ProblemParms.Ki);
% Ei = min(max(U(1),th_opt/(Kgs*Ki)),90/(Kgs*Ki));

X(1,1) = Ei;
X(2,1) = omega_g_filter;

% Only used when m-file is compiled as dll and used directly in Hawc2
if time > time_old
    Ts = time - time_old;
    time_old = time;
    U_old = U;
    X_old = X;
end

% Only used in Simulink
if nargout == 1
    U_old = [U_old;X_old;time_old;Ts];
end